import pandas as pd


def compute_levels_count_and_pct(df: pd.DataFrame,
                                 categorical_column: str) -> pd.DataFrame:
    """Computes categorical levels count and percentage and
    returns a `pd.DataFrame`
    """
    categorical_levels_count = (
        df[categorical_column]
        .value_counts(dropna=False, sort=False)
        .reset_index()
        .rename(columns={categorical_column: f'{categorical_column}_count'})
    )

    categorical_levels_pct = (
        df[categorical_column]
        .value_counts(normalize=True, dropna=False, sort=False)
        .reset_index()
        .rename(columns={categorical_column: f'{categorical_column}_norm'})
    )

    return (
        pd
        .concat([categorical_levels_count,
                 categorical_levels_pct.drop(columns='index')], axis=1)
    )



import shap
import warnings
import pandas as pd

from typing import List
from typing_extensions import Protocol
from shap.common import SHAPError
from sklearn.pipeline import Pipeline


class ScikitModel(Protocol):
    """Scikit model typing
    """
    def fit(self, X, y, verbose): ...
    def predict_proba(self, X) -> object: ...


def cat_features_fillna(df: pd.DataFrame,
                        cat_features: List[str]) -> pd.DataFrame:
    """Fills NA values for each column in `cat_features` for
    `df` dataframe
    """
    df_copy = df.copy()

    for cat in cat_features:
        try:
            df_copy[cat] = (
                df_copy[cat].cat.add_categories('UNKNOWN').fillna('UNKNOWN')
            )

        except AttributeError:
            # The dtype is object instead of category
            df_copy[cat] = df_copy[cat].fillna('UNKNOWN')

    return df_copy


def explainer_plots(model: ScikitModel,
                    X_train: pd.DataFrame,
                    X_test: pd.DataFrame,
                    column_names: List[str]) -> None:
    """Shap plots depending on what model is passed
    in `model` parameter

    `shap.TreeExplainer` by default and if not it uses
    `KernelExplainer`

    Also provides compatibility with sklearn pipelines
    """
    if isinstance(model, Pipeline):
        X_train_to_shap = model[:-1].transform(X_train)
        X_test_to_shap = model[:-1].transform(X_test)
        model = model.steps[-1][1]
    else:
        X_train_to_shap = X_train.copy()
        X_test_to_shap = X_test.copy()

    with warnings.catch_warnings():
        # Some `shap` warnings are not useful for this implementation
        warnings.simplefilter("ignore")
        try:
            explainer = shap.TreeExplainer(
                model=model,
                feature_perturbation='tree_path_dependent'
            )

            shap_values_arguments = dict(X=X_test_to_shap)
        except SHAPError:
            def model_predict(data_array):
                data_frame = pd.DataFrame(data_array, columns=column_names)
                return model.predict_proba(data_frame)[:, 1]

            explainer = shap.KernelExplainer(model=model_predict,
                                             data=shap.sample(X_train_to_shap,
                                                              100),
                                             link='logit')

            shap_values_arguments = dict(X=X_test_to_shap, l1_reg='aic')

    shap_values = explainer.shap_values(**shap_values_arguments)

    shap.summary_plot(shap_values,
                      X_train,
                      plot_type='bar',
                      title='Most Discriminative Features')



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pydrift', 'pydrift.test']

package_data = \
{'': ['*'], 'pydrift': ['catboost_info/*', 'catboost_info/learn/*', 'core/*']}

install_requires = \
['catboost>=0.23,<0.24',
 'coveralls>=2.0.0,<3.0.0',
 'flake8>=3.8.1,<4.0.0',
 'jupyter>=1.0.0,<2.0.0',
 'missingno>=0.4.2,<0.5.0',
 'pandas>=1.0.3,<2.0.0',
 'plotly_express>=0.4.1,<0.5.0',
 'pre-commit>=2.4.0,<3.0.0',
 'pytest>=5.4.2,<6.0.0',
 'shap>=0.35.0,<0.36.0',
 'sklearn>=0.0,<0.1',
 'sphinx>=3.0.3,<4.0.0',
 'sphinx_press_theme>=0.5.1,<0.6.0',
 'typing-extensions>=3.7.4,<4.0.0']

setup_kwargs = {
    'name': 'pydrift',
    'version': '0.1.5',
    'description': 'How do we measure the degradation of a machine learning process? Why does the performance of our predictive models decrease? Maybe it is that a data source has changed (one or more variables) or maybe what changes is the relationship of these variables with the target we want to predict. `pydrift` tries to facilitate this task to the data scientist, performing this kind of checks and somehow measuring that degradation.',
    'long_description': None,
    'author': 'sergiocalde94',
    'author_email': 'sergiocalde94@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
